"use strict";

var debug, hasErrors, main, parse, runOptions, runner, startRuntime, subprocess, testStatusText, testsuite;
testsuite = require('./testsuite');
runner = require('./runner');
subprocess = require('./subprocess');
debug = require('debug')('fbp-spec:cli'); //# Main

parse = function parse(args) {
  var program;
  program = require('commander');
  program.arguments('<suites>').action(function (suites) {
    return program.suites = suites;
  }).option('--address <URL>', 'Address of runtime to connect to', String, 'ws://localhost:3569').option('--secret <secret>', 'Runtime secret', String, null).option('--command <command>', 'Command to launch runtime under test', String, null).option('--start-timeout <seconds>', 'Time to wait for runtime to start', Number, 10).option('--command-timeout <seconds>', 'Max time for a FBP command', Number, 3).parse(process.argv);
  return program;
};

startRuntime = function startRuntime(options, callback) {
  var subprocessOptions;

  if (!options.command) {
    // we're not responsible for starting it
    callback(null);
    return null;
  }

  subprocessOptions = {};
  return subprocess.start(options.command, subprocessOptions, callback);
};

hasErrors = function hasErrors(suites) {
  var c, failures, i, j, len, len1, ref, s;
  failures = 0;

  for (i = 0, len = suites.length; i < len; i++) {
    s = suites[i];
    ref = s.cases;

    for (j = 0, len1 = ref.length; j < len1; j++) {
      c = ref[j];

      if (c.error) {
        failures += 1;
      }
    }
  }

  return failures > 0;
};

runOptions = function runOptions(options, onUpdate, callback) {
  var child, cleanReturn, def, ru, runnerOptions, suites;
  suites = [];

  if (options.suites) {
    suites = testsuite.getSuitesSync(options.suites);
  }

  child = null;

  cleanReturn = function cleanReturn(err) {
    if (child) {
      child.kill();
    }

    return callback(err, suites);
  };

  def = {
    protocol: 'websocket',
    address: options.address,
    secret: options.secret || ''
  };
  debug('runtime info', def);
  runnerOptions = {
    connectTimeout: options.startTimeout * 1000,
    commandTimeout: options.commandTimeout * 1000
  };
  ru = new runner.Runner(def, runnerOptions);
  return child = startRuntime(options, function (err) {
    if (err) {
      cleanReturn(err);
    }

    return ru.connect(function (err) {
      if (err) {
        cleanReturn(err);
      } // TODO: move this into runAll??


      return runner.getComponentSuites(ru, function (err, componentSuites) {
        if (err) {
          cleanReturn(err);
        }

        suites = suites.concat(componentSuites);
        return runner.runAll(ru, suites, onUpdate, function (err) {
          if (err) {
            cleanReturn(err);
          }

          return ru.disconnect(function (err) {
            return cleanReturn(err);
          });
        });
      });
    });
  });
};

testStatusText = function testStatusText(suites) {
  var c, i, ident, j, len, len1, ref, res, results, s;
  results = [];
  ident = '  ';

  for (i = 0, len = suites.length; i < len; i++) {
    s = suites[i];
    ref = s.cases;

    for (j = 0, len1 = ref.length; j < len1; j++) {
      c = ref[j];

      if (c.passed == null || c.shown) {
        continue;
      }

      if (!s.titleshown) {
        results.push("".concat(s.name));
      }

      s.titleshown = true;
      c.shown = true; // bit hacky, mutates suites

      res = c.passed ? '✓' : "\u2717 Error: ".concat(c.error);

      if (c.skip) {
        res = "SKIP: ".concat(c.skip);
      }

      results.push("".concat(ident).concat(c.name, "\n").concat(ident + ident).concat(c.assertion, ": ").concat(res));
    }
  }

  return results;
};

main = function main() {
  var onUpdate, options;
  options = parse(process.argv);

  onUpdate = function onUpdate(suites) {
    var r;
    r = testStatusText(suites);
    return console.log(r.join('\n'));
  };

  return runOptions(options, onUpdate, function (err, suites) {
    var exitStatus;

    if (err) {
      throw err;
    }

    exitStatus = hasErrors(suites) ? 2 : 0;
    return process.exit(exitStatus);
  });
};

exports.main = main;