"use strict";

// FBP protocol dependent code
var Promise, common, debug, fbpGraph, getComponents;
fbpGraph = require('fbp-graph');
common = require('./common');
debug = require('debug')('fbp-spec:protocol');
Promise = require('bluebird');

exports.sendGraph = function (client, graph, callback) {
  var graphId, main;
  main = false; // this is a component?

  if (!graph) {
    return callback(new Error("Graph not defined"));
  }

  graphId = graph.name || graph.properties.id;

  if (!graphId) {
    graphId = "fixture.".concat(common.randomString(10));
  }

  graph.name = graphId;

  if (!(graph instanceof fbpGraph.Graph)) {
    // fbp-client operates on fbp-graph instances
    fbpGraph.graph.loadJSON(graph, function (err, g) {
      if (err) {
        return callback(err);
      }

      return exports.sendGraph(client, g, callback);
    });
    return;
  }

  debug('sendgraph', graphId);
  Promise.resolve().then(function () {
    return client.protocol.graph.send(graph, main);
  }).then(function () {
    return graphId;
  }).nodeify(callback);
};

exports.startNetwork = function (client, graphId, callback) {
  debug('startnetwork', graphId);
  Promise.resolve().then(function () {
    return client.protocol.network.start({
      graph: graphId
    });
  }).nodeify(callback);
};

exports.stopNetwork = function (client, graphId, callback) {
  debug('stopnetwork', graphId);
  Promise.resolve().then(function () {
    return client.protocol.network.stop({
      graph: graphId
    });
  }).nodeify(callback);
};

exports.sendPackets = function (client, graphId, packets, callback) {
  debug('sendpackets', graphId, packets);
  Promise.all(Object.keys(packets).map(function (port) {
    return client.protocol.runtime.packet({
      event: 'data',
      port: port,
      payload: packets[port],
      graph: graphId
    });
  })).nodeify(callback);
};

exports.getComponents = getComponents = function getComponents(client, callback) {
  debug('get components');
  Promise.resolve().then(function () {
    return client.protocol.component.list();
  }).then(function (componentList) {
    var component, components, i, len;
    components = {};

    for (i = 0, len = componentList.length; i < len; i++) {
      component = componentList[i];
      components[component.name] = component;
    }

    return components;
  }).nodeify(callback);
};

exports.getCapabilities = function (client, callback) {
  var def, ref;
  def = client.definition;

  if (def != null ? (ref = def.capabilities) != null ? ref.length : void 0 : void 0) {
    return callback(null, def.capabilities);
  }

  Promise.resolve().then(function () {
    return client.protocol.runtime.getruntime();
  }).then(function (definition) {
    return definition.capabilities;
  }).nodeify(callback);
};

exports.getComponentTests = function (client, callback) {
  debug('get component tests');
  Promise.resolve().then(function () {
    return client.protocol.component.list();
  }).then(function (components) {
    return Promise.all(components.map(function (component) {
      return client.protocol.component.getsource({
        name: component.name
      });
    }));
  }).then(function (sources) {
    var i, len, name, source, tests;
    tests = {};

    for (i = 0, len = sources.length; i < len; i++) {
      source = sources[i];

      if (!source.tests) {
        continue;
      }

      name = source.library ? "".concat(source.library, "/").concat(source.name) : source.name;
      tests[name] = source.tests;
    }

    return tests;
  }).nodeify(callback);
};