"use strict";

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

var Promise,
    Runner,
    checkResults,
    common,
    debug,
    debugReceivedMessages,
    expectation,
    fbp,
    fbpClient,
    getFixtureGraph,
    loadComponentSuites,
    needsSetup,
    protocol,
    runAll,
    runSuite,
    runTestAndCheck,
    sendMessageAndWait,
    synthesizeTopicFixture,
    testsuite,
    indexOf = [].indexOf;
common = require('./common');
protocol = require('./protocol');
testsuite = require('./testsuite');
expectation = require('./expectation');
fbp = require('fbp');
fbpClient = require('fbp-client');
debug = require('debug')('fbp-spec:runner');
Promise = require('bluebird');

debugReceivedMessages = function debugReceivedMessages(client) {
  var debugReceived;
  debugReceived = require('debug')('fbp-spec:runner:received');
  return client.on('signal', function (_ref) {
    var protocol = _ref.protocol,
        command = _ref.command,
        payload = _ref.payload;
    return debugReceived(protocol, command, payload);
  });
};

synthesizeTopicFixture = function synthesizeTopicFixture(topic, components, callback) {
  var component, graph, j, k, len, len1, port, portName, processName, ref, ref1;
  debug('synthesizing fixture', topic); // export each of the ports for topic component
  // return a FBP graph?

  component = components[topic];

  if (!component) {
    return callback(new Error("Could not find component for topic: ".concat(topic)));
  }

  graph = {
    properties: {},
    inports: {},
    outports: {},
    processes: {},
    connections: []
  };
  processName = 'testee';
  graph.processes[processName] = {
    component: topic
  };
  ref = component.inPorts;

  for (j = 0, len = ref.length; j < len; j++) {
    port = ref[j];
    portName = port.id;
    graph.inports[portName] = {
      process: processName,
      port: portName
    };
  }

  ref1 = component.outPorts;

  for (k = 0, len1 = ref1.length; k < len1; k++) {
    port = ref1[k];
    portName = port.id;
    graph.outports[portName] = {
      process: processName,
      port: portName
    };
  } // Sanity checking if this is usable as a fixture


  if (Object.keys(graph.outports) < 1) {
    return callback(new Error("Component '".concat(topic, "' used as fixture has no outports")));
  }

  if (Object.keys(graph.inports) < 1) {
    return callback(new Error("Component '".concat(topic, "' used as fixture has no inports")));
  }

  return callback(null, graph);
}; // @context should have .components = {} property


getFixtureGraph = function getFixtureGraph(context, suite, callback) {
  var hasComponents, updateComponents; // TODO: follow runtime for component changes

  hasComponents = function hasComponents(s) {
    return s.components != null && _typeof(s.components) === 'object' && Object.keys(s.components).length;
  };

  updateComponents = function updateComponents(cb) {
    if (hasComponents(context)) {
      return cb(null);
    }

    protocol.getComponents(context.client, function (err, components) {
      if (err) {
        return cb(err);
      }

      context.components = components;
      return cb(null);
    });
  };

  updateComponents(function (err) {
    var e, graph;

    if (err) {
      return callback(err);
    }

    if (suite.fixture == null) {
      return synthesizeTopicFixture(suite.topic, context.components, callback);
    } else if (suite.fixture.type === 'json') {
      try {
        graph = JSON.parse(suite.fixture.data);
      } catch (error1) {
        e = error1;
        return callback(e);
      }

      return callback(null, graph);
    } else if (suite.fixture.type === 'fbp') {
      try {
        graph = fbp.parse(suite.fixture.data);
      } catch (error1) {
        e = error1;
        return callback(e);
      }

      if (!graph.properties) {
        graph.properties = {};
      }

      return callback(null, graph);
    } else {
      return callback(new Error("Unknown fixture type ".concat(suite.fixture.type)));
    }
  });
};

sendMessageAndWait = function sendMessageAndWait(client, currentGraph, inputData, expectData, callback) {
  var checkFailure, checkSuccess, observer, sendPackets, signalsToReceived;
  observer = client.observe(function (s) {
    return s.protocol === 'runtime' && s.command === 'packet' && s.payload.graph === currentGraph;
  });

  signalsToReceived = function signalsToReceived(signals) {
    var j, len, received, signal;
    received = {};

    for (j = 0, len = signals.length; j < len; j++) {
      signal = signals[j];
      received[signal.payload.port] = signal.payload.payload;
    }

    return received;
  };

  checkSuccess = function checkSuccess(s) {
    var received, receivedSignals, result;

    if (!(s.protocol === 'runtime' && s.command === 'packet')) {
      // We're only interested in response packets
      return false;
    }

    if (s.payload.graph !== currentGraph) {
      // Check that is for the current graph under test
      return false;
    }

    if (s.payload.event !== 'data') {
      // We're only interested in data IPs, not brackets
      return false;
    } // Get signals received until this message


    receivedSignals = observer.signals.slice(0, observer.signals.indexOf(s) + 1);
    received = signalsToReceived(receivedSignals);
    result = Object.keys(received).length === Object.keys(expectData).length;
    return result;
  };

  checkFailure = function checkFailure(s) {
    if (s.protocol === 'network' && s.command === 'error') {
      if (s.payload.graph && s.payload.graph !== currentGraph) {
        // network:error should imply failed test
        return false;
      }

      return true;
    }

    if (s.protocol === 'network' && s.command === 'processerror') {
      if (s.payload.graph && s.payload.graph !== currentGraph) {
        // network:processerror should imply failed test
        return false;
      }

      return true;
    }

    if (s.protocol === 'runtime' && s.command === 'packet') {
      if (s.payload.graph !== currentGraph) {
        // Output packet, see if it is an unexpected error
        // Check that is for the current graph under test
        return false;
      }

      if (s.payload.event !== 'data') {
        // We're only interested in data IPs, not brackets
        return false;
      }

      if (s.payload.port !== 'error') {
        // We only care for packets to error port
        return false;
      }

      if (typeof expectData.error !== 'undefined') {
        // We only care if spec isn't expecting errors
        return false;
      }

      return true;
    }

    return false;
  }; // send input packets


  sendPackets = Promise.promisify(protocol.sendPackets);
  Promise.resolve().then(function () {
    return sendPackets(client, currentGraph, inputData);
  }).then(function () {
    return observer.until(checkSuccess, checkFailure);
  }).then(function (signals) {
    return signalsToReceived(signals);
  }).nodeify(callback);
};

needsSetup = function needsSetup(suite) {
  var notSkipped;
  notSkipped = suite.cases.filter(function (c) {
    return !c.skip;
  });
  return notSkipped.length > 0;
};

Runner = /*#__PURE__*/function () {
  function Runner(client1) {
    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    _classCallCheck(this, Runner);

    this.client = client1;
    this.currentGraphId = null;
    this.components = {};
    this.parentElement = null;
    this.options = options;

    if (this.options.connectTimeout == null) {
      this.options.connectTimeout = 5 * 1000;
    }

    if (this.options.commandTimeout == null) {
      this.options.commandTimeout = 3 * 1000;
    }
  }

  _createClass(Runner, [{
    key: "prepareClient",
    value: function prepareClient(callback) {
      var _this = this;

      if (this.client.protocol != null && this.client.address != null) {
        // is a runtime definition
        Promise.resolve().then(function () {
          return fbpClient(_this.client, {
            commandTimeout: _this.options.commandTimeout
          });
        }).then(function (client) {
          _this.client = client;

          if (_this.parentElement && client.definition.protocol === 'iframe') {
            // We need to set up the parent element in this case
            client.transport.setParentElement(_this.parentElement);
          }

          return client;
        }).nodeify(callback);
        return;
      } // This is a client instance


      callback(null, this.client);
    } // TODO: check the runtime capabilities before continuing

  }, {
    key: "connect",
    value: function connect(callback) {
      var _this2 = this;

      debug('connect');
      this.prepareClient(function (err) {
        var attempts, isOnline, timeBetweenAttempts, tryConnect;

        if (err) {
          return callback(err);
        }

        debugReceivedMessages(_this2.client);

        _this2.client.on('network', function (_ref2) {
          var command = _ref2.command,
              payload = _ref2.payload;

          if (command === 'output' && payload.message) {
            return console.log(payload.message);
          }
        });

        timeBetweenAttempts = 500;
        attempts = Math.floor(_this2.options.connectTimeout / timeBetweenAttempts);

        isOnline = function isOnline() {
          var connected;
          connected = _this2.client.isConnected();

          if (connected) {
            return Promise.resolve();
          } else {
            return Promise.reject(new Error('Not connected to runtime'));
          }
        };

        tryConnect = function tryConnect() {
          debug('trying to connect');
          return _this2.client.connect();
        };

        return common.retryUntil(tryConnect, isOnline, timeBetweenAttempts, attempts).then(function () {
          return _this2.checkCapabilities(['protocol:graph', 'protocol:network', 'protocol:runtime']);
        }).nodeify(callback);
      });
    }
  }, {
    key: "disconnect",
    value: function disconnect(callback) {
      var _this3 = this;

      var ref;
      debug('disconnect');

      if (!((ref = this.client) != null ? ref.isConnected() : void 0)) {
        return callback();
      }

      Promise.resolve().then(function () {
        return _this3.client.disconnect();
      }).nodeify(callback);
    }
  }, {
    key: "checkCapabilities",
    value: function checkCapabilities(capabilities) {
      var capability, j, len, ref, ref1;

      if (!this.client.isConnected()) {
        return Promise.reject(new Error('Not connected to runtime'));
      }

      if (!((ref = this.client.definition) != null ? (ref1 = ref.capabilities) != null ? ref1.length : void 0 : void 0)) {
        return Promise.reject(new Error('Runtime provides no capabilities'));
      }

      for (j = 0, len = capabilities.length; j < len; j++) {
        capability = capabilities[j];

        if (this.client.definition.capabilities.indexOf(capability) === -1) {
          return Promise.reject(new Error("Runtime doesn't provide ".concat(capability)));
        }
      }

      return Promise.resolve();
    }
  }, {
    key: "setupSuite",
    value: function setupSuite(suite, callback) {
      var _this4 = this;

      debug('setup suite', "\"".concat(suite.name, "\""));

      if (!needsSetup(suite)) {
        return callback(null);
      }

      if (!this.client.isConnected()) {
        return callback(new Error('Disconnected from runtime'));
      }

      getFixtureGraph(this, suite, function (err, graph) {
        if (err) {
          return callback(err);
        }

        protocol.sendGraph(_this4.client, graph, function (err, graphId) {
          _this4.currentGraphId = graphId;

          if (err) {
            return callback(err);
          }

          return protocol.startNetwork(_this4.client, graphId, callback);
        });
      });
    }
  }, {
    key: "teardownSuite",
    value: function teardownSuite(suite, callback) {
      debug('teardown suite', "\"".concat(suite.name, "\""));

      if (!needsSetup(suite)) {
        return callback(null);
      }

      if (!this.client.isConnected()) {
        return callback(new Error('Disconnected from runtime'));
      } // FIXME: also remove the graph. Ideally using a 'destroy' message in FBP protocol


      protocol.stopNetwork(this.client, this.currentGraphId, callback);
    }
  }, {
    key: "runTest",
    value: function runTest(testcase, callback) {
      var _this5 = this;

      var expects, i, inputs, j, ref, sendWait, sequence;
      debug('runtest', "\"".concat(testcase.name, "\""));

      if (!this.client.isConnected()) {
        return callback(new Error('Disconnected from runtime'));
      } // XXX: normalize and validate in testsuite.coffee instead?


      inputs = common.isArray(testcase.inputs) ? testcase.inputs : [testcase.inputs];
      expects = common.isArray(testcase.expect) ? testcase.expect : [testcase.expect];
      sequence = [];

      for (i = j = 0, ref = inputs.length; 0 <= ref ? j < ref : j > ref; i = 0 <= ref ? ++j : --j) {
        sequence.push({
          inputs: inputs[i],
          expect: expects[i]
        });
      }

      sendWait = function sendWait(data, cb) {
        return sendMessageAndWait(_this5.client, _this5.currentGraphId, data.inputs, data.expect, cb);
      };

      common.asyncSeries(sequence, sendWait, function (err, actuals) {
        if (err) {
          return callback(err);
        }

        actuals.forEach(function (r, idx) {
          return sequence[idx].actual = r;
        });
        return callback(null, sequence);
      });
    }
  }]);

  return Runner;
}(); // TODO: should this go into expectation?


checkResults = function checkResults(results) {
  var actuals, errors, expects, failed, failures, index, result;
  actuals = results.filter(function (r) {
    return r.actual != null;
  });
  expects = results.filter(function (r) {
    return r.expect != null;
  });

  if (actuals.length < expects.length) {
    result = {
      passed: false,
      error: new Error("Only got ".concat(actual.length, " output messages out of ").concat(expect.length))
    };
    return result;
  }

  results = results.map(function (res) {
    var e;
    res.error = null;

    try {
      expectation.expect(res.expect, res.actual);
    } catch (error1) {
      e = error1; // FIXME: only catch actual AssertionErrors

      res.error = e;
    }

    return res;
  });
  failures = results.filter(function (r) {
    return r.error;
  });

  if (failures.length === 0) {
    result = {
      passed: true
    };
  } else {
    if (expects.length === 1) {
      result = {
        error: failures[0].error
      };
    } else if (expects.length > 1 && failures.length === 1) {
      index = results.findIndex(function (r) {
        return r.error;
      });
      failed = results[index];
      result = {
        error: new Error("Expectation ".concat(index, " of sequence failed: ").concat(failed.error.message))
      };
    } else {
      errors = results.map(function (r) {
        var ref;
        return ((ref = r.error) != null ? ref.message : void 0) || '';
      });
      result = {
        error: new Error("Multiple failures in sequence: ".concat(errors))
      };
    }
  }

  return result;
};

runTestAndCheck = function runTestAndCheck(runner, testcase, callback) {
  var expects, inputs;

  if (testcase.skip) {
    return callback(null, {
      passed: true
    });
  } // TODO: pass some skipped state? its indirectly in .skip though
  // XXX: normalize and validate in testsuite.coffee instead?


  inputs = common.isArray(testcase.inputs) ? testcase.inputs : [testcase.inputs];
  expects = common.isArray(testcase.expect) ? testcase.expect : [testcase.expect];

  if (inputs.length !== expects.length) {
    return callback(null, {
      passed: false,
      error: new Error("Test sequence length mismatch. Got ".concat(inputs.length, " inputs and ").concat(expects.length, " expectations"))
    });
  }

  runner.runTest(testcase, function (err, results) {
    var result;

    if (err) {
      // Map error to a test failure
      result = {
        passed: false,
        error: err
      };
      return callback(null, result);
    }

    result = checkResults(results);

    if (result.error) {
      result.passed = false;
    }

    return callback(null, result);
  });
};

runSuite = function runSuite(runner, suite, runTest, callback) {
  if (suite.skip) {
    return callback(null, suite);
  } // TODO: pass some skipped state? its indirectly in .skip though


  runner.setupSuite(suite, function (err) {
    debug('setup suite', err);

    if (err) {
      return callback(err, suite);
    }

    return common.asyncSeries(suite.cases, runTest, function (err) {
      debug('testrun complete', err);
      return runner.teardownSuite(suite, function (err) {
        debug('teardown suite', err);
        return callback(err, suite);
      });
    });
  });
};

exports.getComponentSuites = function (runner, callback) {
  protocol.getCapabilities(runner.client, function (err, caps) {
    if (err) {
      return callback(err);
    }

    if (indexOf.call(caps, 'component:getsource') < 0) {
      return callback(null, []);
    }

    return protocol.getComponentTests(runner.client, function (err, tests) {
      var suites;

      if (err) {
        return callback(err);
      }

      suites = loadComponentSuites(tests);
      debug('get component suites', tests.length, suites.length);
      return callback(null, suites);
    });
  });
};

loadComponentSuites = function loadComponentSuites(componentTests) {
  var e, name, ss, suites, tests;
  suites = [];

  for (name in componentTests) {
    tests = componentTests[name];

    try {
      ss = testsuite.loadYAML(tests);
      suites = suites.concat(ss);
    } catch (error1) {
      e = error1; // ignore, could be non fbp-spec test
      // TODO: include tests type in FBP protocol, so we know whether this is error or legit

      continue;
    }
  }

  return suites;
}; // will update each of the testcases in @suites
// with .passed and .error states as tests are ran


runAll = function runAll(runner, suites, updateCallback, doneCallback) {
  var runOneSuite, runTest, s;

  runTest = function runTest(testcase, callback) {
    var done;

    done = function done(error) {
      updateCallback(suites);
      return callback(error);
    };

    return runTestAndCheck(runner, testcase, function (err, results) {
      var key, val;

      for (key in results) {
        val = results[key];
        testcase[key] = val;
      }

      if (testcase.error) {
        testcase.error = testcase.error.message;
      }

      debug('ran test', '"testcase.name"', testcase.passed, err);
      return done(null); // ignore error to not bail out early
    });
  };

  runOneSuite = function runOneSuite(suite, cb) {
    return runSuite(runner, suite, runTest, cb);
  };

  debug('running suites', function () {
    var j, len, results1;
    results1 = [];

    for (j = 0, len = suites.length; j < len; j++) {
      s = suites[j];
      results1.push(s.name);
    }

    return results1;
  }());
  common.asyncSeries(suites, runOneSuite, function (err) {
    return doneCallback(err);
  });
};

exports.Runner = Runner;
exports.runAll = runAll;
exports.runTestAndCheck = runTestAndCheck;